"use strict";

/* global Components, ExtensionAPI */
let Cc = Components.classes;
let Cu = Components.utils;
let Ci = Components.interfaces;
Cu.import("resource://gre/modules/Services.jsm");

const {EventManager} = ExtensionCommon;

// Implements an experimental extension to the notifications api

Cu.import("resource://gre/modules/EventEmitter.jsm");
Cu.import("resource://gre/modules/BrowserUtils.jsm");

class NotificationPrompt {
  constructor(extension, notificationsMap, id, options) {
    this.notificationsMap = notificationsMap;
    this.id = id;
    this.options = options;

    const browserWin = Services.wm.getMostRecentWindow("navigator:browser");
    let buttonsOutput = [];
    if (options.buttons) {
      const addButton = (button) => {
        let buttonIndex = buttonsOutput.length;
        buttonsOutput.push({
          label: button.title,
          callback: () => {
            this.handleEvent("buttonClicked", {
              notificationId: id,
              buttonIndex
            });
          }
        });
      };
      for (let button of options.buttons) {
        addButton(button);
      }
    }
    this.box = browserWin.document.getElementById("global-notificationbox");
    let outputMessage = options.message;
    if (options.moreInfo) {
        let mainMessage = "%S %S";
        let text = options.moreInfo.title || "Learn more";
        let link = browserWin.document.createElement("label");
        link.className = "text-link";
        link.setAttribute("useoriginprincipal", true);
        link.setAttribute("href", options.moreInfo.url);
        link.textContent = text;
        outputMessage = BrowserUtils.getLocalizedFragment(browserWin.document, mainMessage, outputMessage, link);
    }
    this.box.appendNotification(outputMessage, id, null, this.box.PRIORITY_INFO_HIGH,
      buttonsOutput);
  }

  clear() {
    this.box.getNotificationWithValue(this.id).close();
    this.notificationsMap.delete(this.id);
  }

  handleEvent(event, data) {
    this.notificationsMap.emit(event, data);
  }
}

var notifications = class notifications extends ExtensionAPI {
  constructor(extension) {
    super(extension);

    this.nextId = 0;
    this.notificationsMap = new Map();
    EventEmitter.decorate(this.notificationsMap);
  }

  onShutdown() {
    for (let notification of this.notificationsMap.values()) {
      notification.clear();
    }
  }

  getAPI(context) {
    let {extension} = context;
    let notificationsMap = this.notificationsMap;

    return {
      experiments: {
        notifications: {
          clear: function(notificationId) {
            if (notificationsMap.has(notificationId)) {
              notificationsMap.get(notificationId).clear();
            }
          },
          create: (notificationId, options) => {
            if (!notificationId) {
              notificationId = String(this.nextId++);
            }
  
            if (notificationsMap.has(notificationId)) {
              notificationsMap.get(notificationId).clear();
            }
  
            let notification;
            if (options.type === "prompt") {
              notification = new NotificationPrompt(extension, notificationsMap, notificationId, options);
            } else {
             // Normal notices here unsupported in experiment
            }
            notificationsMap.set(notificationId, notification);
  
            return notificationId;
          },
  
          onButtonClicked: new EventManager(
            context,
            "notifications.onButtonClicked",
            fire => {
              let listener = (event, data) => {
                fire.async(data);
              };
  
              notificationsMap.on("buttonClicked", listener);
              return () => {
                notificationsMap.off("buttonClicked", listener);
              };
            },
          ).api(),
        },
      },
    };
  }
};
